#!/usr/bin/env python
# -*- coding: utf_8 -*-

__title__ = 'njsscan'
__authors__ = 'Ajin Abraham'
__copyright__ = 'Copyright 2020 Ajin Abraham, OpenSecurity'
__version__ = '0.1.9'
__version_info__ = tuple(int(i) for i in __version__.split('.'))
__all__ = [
    '__title__',
    '__authors__',
    '__copyright__',
    '__version__',
    '__version_info__',
]
